@@ -41,6 +41,53 @@
  * INJECT CODE
  ********************************************************************/
 
+// @snippet qopenglcontext-versionfunctions
+
+// %CPPSELF.%FUNCTION_NAME(%1, %2); Pretend to shiboken we call the function
+
+// Import QtOpenGLFunctions and call the factory function
+// QOpenGLVersionFunctionsFactory.get()
+PyObject *module = PyImport_ImportModule("PySide2.QtOpenGLFunctions");
+if (module == nullptr)
+    return nullptr;
+PyObject *loc = PyModule_GetDict(module);
+static PyObject *const factoryName =
+    Shiboken::String::createStaticString("QOpenGLVersionFunctionsFactory");
+auto factory = PyDict_GetItem(loc, factoryName);
+if (factory == nullptr)
+    return nullptr;
+
+static PyObject *const getMethod = Shiboken::String::createStaticString("get");
+%PYARG_0 = PyObject_CallMethodObjArgs(factory, getMethod, pyArgs[0], %PYSELF,
+                                       nullptr);
+// @snippet qopenglcontext-versionfunctions
+
+// @snippet glgetshadersource
+GLsizei bufSize = 4096;
+GLsizei length = bufSize - 1;
+QByteArray buffer;
+for (; length == bufSize - 1; bufSize += 4096) {
+    buffer.resize(qsizetype(bufSize));
+    %CPPSELF->%FUNCTION_NAME(%1, bufSize, &length, buffer.data());
+    if (%CPPSELF->glGetError() != GL_NO_ERROR) {
+        buffer.clear();
+        break;
+    }
+}
+auto *data = buffer.constData();
+%PYARG_0 = %CONVERTTOPYTHON[char *](data);
+// @snippet glgetshadersource
+
+// @snippet glshadersource
+const QByteArray buffer = %2.toUtf8();
+const char *sources[] = {buffer.constData()};
+%CPPSELF->%FUNCTION_NAME(%1, 1, sources, nullptr);
+// @snippet glshadersource
+
+// @snippet glgetstring-return
+%PYARG_0 = %CONVERTTOPYTHON[const char *](%0);
+// @snippet glgetstring-return
+
 // @snippet qtransform-quadtoquad
 QTransform _result;
 if (QTransform::quadToQuad(%1, %2, _result)) {
@@ -78,9 +125,7 @@ QBitmap %0 = QBitmap::fromData(%1, buffer, %3);
 // @snippet qbitmap-fromdata
 
 // @snippet qtextline-cursortox
-%BEGIN_ALLOW_THREADS
 %RETURN_TYPE %0 = %CPPSELF->::%TYPE::%FUNCTION_NAME(&%1, %2);
-%END_ALLOW_THREADS
 %PYARG_0 = PyTuple_New(2);
 PyTuple_SET_ITEM(%PYARG_0, 0, %CONVERTTOPYTHON[%RETURN_TYPE](%0));
 PyTuple_SET_ITEM(%PYARG_0, 1, %CONVERTTOPYTHON[%ARG1_TYPE](%1));
@@ -145,6 +190,16 @@ for (int i = 0, i_max = %CPPSELF.count(); i < i_max; ++i){
 %0 = new %TYPE(QPixmap::fromImage(%1));
 // @snippet qpixmap
 
+// @snippet qimage-decref-image-data
+static void imageDecrefDataHandler(void *data)
+{
+    // Avoid "Python memory allocator called without holding the GIL"
+    auto state = PyGILState_Ensure();
+    Py_DECREF(reinterpret_cast<PyObject *>(data));
+    PyGILState_Release(state);
+}
+// @snippet qimage-decref-image-data
+
 // @snippet qimage-constbits
 %PYARG_0 = Shiboken::Buffer::newObject(%CPPSELF.%FUNCTION_NAME(), %CPPSELF.byteCount());
 // @snippet qimage-constbits
@@ -444,9 +499,7 @@ PyTuple_SET_ITEM(%PYARG_0, 1, %CONVERTTOPYTHON[%ARG1_TYPE](%1));
 // @snippet qclipboard-text
 
 // @snippet qpainter-drawpolygon
-%BEGIN_ALLOW_THREADS
 %CPPSELF.%FUNCTION_NAME(%1.data(), %1.size(), %2);
-%END_ALLOW_THREADS
 // @snippet qpainter-drawpolygon
 
 // @snippet qmatrix-map-point
