@@ -30,6 +30,7 @@
 
 #include "codemodel.h"
 
+#include <sourcelocation.h>
 #include <clangparser/clangutils.h>
 
 #include <algorithm>
@@ -40,7 +41,7 @@
 #include <QtCore/QStack>
 
 // Predicate to find an item by name in a list of QSharedPointer<Item>
-template <class T> class ModelItemNamePredicate : public std::unary_function<bool, QSharedPointer<T> >
+template <class T> class ModelItemNamePredicate
 {
 public:
     explicit ModelItemNamePredicate(const QString &name) : m_name(name) {}
@@ -91,6 +92,8 @@ static CodeModelItem findRecursion(const ScopeModelItem &scope,
             return es;
         if (TypeDefModelItem tp = scope->findTypeDef(nameSegment))
             return tp;
+        if (TemplateTypeAliasModelItem tta = scope->findTemplateTypeAlias(nameSegment))
+            return tta;
         return CodeModelItem();
     }
     if (auto nestedClass = scope->findClass(nameSegment))
@@ -191,13 +194,29 @@ TypeInfo TypeInfo::resolveType(CodeModelItem __item, TypeInfo const &__type, con
         return resolveType(nextItem, combined, __scope);
     }
 
+    if (TemplateTypeAliasModelItem templateTypeAlias = qSharedPointerDynamicCast<_TemplateTypeAliasModelItem>(__item)) {
+
+        TypeInfo combined = TypeInfo::combine(templateTypeAlias->type(), otherType);
+        // For the alias "template<typename T> using QList = QVector<T>" with
+        // other="QList<int>", replace the instantiations to obtain "QVector<int>".
+        auto aliasInstantiations = templateTypeAlias->type().instantiations();
+        auto concreteInstantiations = otherType.instantiations();
+        const int count = qMin(aliasInstantiations.size(), concreteInstantiations.size());
+        for (int i = 0; i < count; ++i)
+            aliasInstantiations[i] = concreteInstantiations[i];
+        combined.setInstantiations(aliasInstantiations);
+        const CodeModelItem nextItem = __scope->model()->findItem(combined.qualifiedName(), __scope);
+        if (!nextItem)
+            return combined;
+        return resolveType(nextItem, combined, __scope);
+    }
+
     return otherType;
 }
 
 // Handler for clang::parseTemplateArgumentList() that populates
 // TypeInfo::m_instantiations
-class TypeInfoTemplateArgumentHandler :
-    public std::binary_function<void, int, const QStringRef &>
+class TypeInfoTemplateArgumentHandler
 {
 public:
     explicit TypeInfoTemplateArgumentHandler(TypeInfo *t)
@@ -393,6 +412,33 @@ void TypeInfo::simplifyStdType()
     }
 }
 
+void TypeInfo::formatTypeSystemSignature(QTextStream &str) const
+{
+    if (m_constant)
+        str << "const ";
+    str << m_qualifiedName.join(QLatin1String("::"));
+    switch (m_referenceType) {
+    case NoReference:
+        break;
+    case LValueReference:
+        str << '&';
+        break;
+    case RValueReference:
+        str << "&&";
+        break;
+    }
+    for (auto i : m_indirections) {
+        switch (i) {
+        case Indirection::Pointer:
+            str << '*';
+            break;
+        case Indirection::ConstPointer:
+            str << "* const";
+            break;
+        }
+    }
+}
+
 #ifndef QT_NO_DEBUG_STREAM
 template <class It>
 void formatSequence(QDebug &d, It i1, It i2, const char *separator=", ")
@@ -563,6 +609,21 @@ void _CodeModelItem::setEndPosition(int line, int column)
     m_endColumn = column;
 }
 
+SourceLocation _CodeModelItem::sourceLocation() const
+{
+    return SourceLocation(m_fileName, m_startLine);
+}
+
+_ScopeModelItem::_ScopeModelItem(CodeModel *model, int kind)
+    : _CodeModelItem(model, kind)
+{
+}
+
+_ScopeModelItem::_ScopeModelItem(CodeModel *model, const QString &name, int kind)
+    : _CodeModelItem(model, name, kind)
+{
+}
+
 #ifndef QT_NO_DEBUG_STREAM
 template <class It>
 static void formatPtrSequence(QDebug &d, It i1, It i2, const char *separator=", ")
@@ -613,6 +674,9 @@ void _CodeModelItem::formatKind(QDebug &d, int k)
     case Kind_TypeDef:
         d << "TypeDefModelItem";
         break;
+    case Kind_TemplateTypeAlias:
+        d << "TemplateTypeAliasModelItem";
+        break;
     default:
         d << "CodeModelItem";
         break;
@@ -688,6 +752,16 @@ bool _ClassModelItem::extendsClass(const QString &name) const
     return false;
 }
 
+_ClassModelItem::_ClassModelItem(CodeModel *model, int kind)
+    : _ScopeModelItem(model, kind)
+{
+}
+
+_ClassModelItem::_ClassModelItem(CodeModel *model, const QString &name, int kind)
+    : _ScopeModelItem(model, name, kind)
+{
+}
+
 void _ClassModelItem::setClassType(CodeModel::ClassType type)
 {
     m_classType = type;
@@ -703,6 +777,16 @@ void _ClassModelItem::addPropertyDeclaration(const QString &propertyDeclaration)
     m_propertyDeclarations << propertyDeclaration;
 }
 
+bool _ClassModelItem::isEmpty() const
+{
+    return _ScopeModelItem::isEmpty() && m_propertyDeclarations.isEmpty();
+}
+
+bool _ClassModelItem::isTemplate() const
+{
+    return !m_templateParameters.isEmpty();
+}
+
 #ifndef QT_NO_DEBUG_STREAM
 template <class List>
 static void formatModelItemList(QDebug &d, const char *prefix, const List &l,
@@ -735,6 +819,8 @@ void _ClassModelItem::formatDebug(QDebug &d) const
     }
     formatModelItemList(d, ", templateParameters=", m_templateParameters);
     formatScopeItemsDebug(d);
+    if (!m_propertyDeclarations.isEmpty())
+        d << ", Properties=" << m_propertyDeclarations;
 }
 #endif // !QT_NO_DEBUG_STREAM
 
@@ -776,6 +862,11 @@ void _ScopeModelItem::addTypeDef(const TypeDefModelItem &item)
     m_typeDefs.append(item);
 }
 
+void _ScopeModelItem::addTemplateTypeAlias(const TemplateTypeAliasModelItem &item)
+{
+    m_templateTypeAliases.append(item);
+}
+
 void _ScopeModelItem::addEnum(const EnumModelItem &item)
 {
     m_enums.append(item);
@@ -786,11 +877,55 @@ void _ScopeModelItem::appendScope(const _ScopeModelItem &other)
     m_classes += other.m_classes;
     m_enums += other.m_enums;
     m_typeDefs += other.m_typeDefs;
+    m_templateTypeAliases += other.m_templateTypeAliases;
     m_variables += other.m_variables;
     m_functions += other.m_functions;
     m_enumsDeclarations += other.m_enumsDeclarations;
 }
 
+bool _ScopeModelItem::isEmpty() const
+{
+    return m_classes.isEmpty() && m_enums.isEmpty()
+        && m_typeDefs.isEmpty() && m_templateTypeAliases.isEmpty()
+        && m_variables.isEmpty() && m_functions.isEmpty()
+        && m_enumsDeclarations.isEmpty();
+}
+
+/* This function removes MSVC export declarations of non-type template
+ * specializations (see below code from photon.h) for which
+ * clang_isCursorDefinition() returns true, causing them to be added as
+ * definitions of empty classes shadowing the template definition depending
+ *  on QHash seed values.
+
+template <int N> class Tpl
+{
+public:
+...
+};
+
+#ifdef WIN32
+template class LIBSAMPLE_EXPORT Tpl<54>;
+*/
+void _ScopeModelItem::purgeClassDeclarations()
+{
+    for (int i = m_classes.size() - 1; i >= 0; --i) {
+        auto klass = m_classes.at(i);
+        // For an empty class, check if there is a matching template
+        // definition, and remove it if this is the case.
+        if (!klass->isTemplate() && klass->isEmpty()) {
+            const QString definitionPrefix = klass->name() + QLatin1Char('<');
+            const bool definitionFound =
+                std::any_of(m_classes.cbegin(), m_classes.cend(),
+                            [definitionPrefix] (const ClassModelItem &c) {
+                    return c->isTemplate() && !c->isEmpty()
+                        && c->name().startsWith(definitionPrefix);
+            });
+            if (definitionFound)
+                m_classes.removeAt(i);
+        }
+    }
+}
+
 #ifndef QT_NO_DEBUG_STREAM
 template <class Hash>
 static void formatScopeHash(QDebug &d, const char *prefix, const Hash &h,
@@ -830,6 +965,7 @@ void _ScopeModelItem::formatScopeItemsDebug(QDebug &d) const
     formatScopeList(d, ", classes=", m_classes, "\n", true);
     formatScopeList(d, ", enums=", m_enums, "\n", true);
     formatScopeList(d, ", aliases=", m_typeDefs, "\n", true);
+    formatScopeList(d, ", template type aliases=", m_templateTypeAliases, "\n", true);
     formatScopeList(d, ", functions=", m_functions, "\n", true);
     formatScopeList(d, ", variables=", m_variables);
 }
@@ -845,7 +981,7 @@ namespace {
 // Predicate to match a non-template class name against the class list.
 // "Vector" should match "Vector" as well as "Vector<T>" (as seen for methods
 // from within the class "Vector").
-class ClassNamePredicate : public std::unary_function<bool, ClassModelItem>
+class ClassNamePredicate
 {
 public:
     explicit ClassNamePredicate(const QString &name) : m_name(name) {}
@@ -881,6 +1017,11 @@ TypeDefModelItem _ScopeModelItem::findTypeDef(const QString &name) const
     return findModelItem(m_typeDefs, name);
 }
 
+TemplateTypeAliasModelItem _ScopeModelItem::findTemplateTypeAlias(const QString &name) const
+{
+    return findModelItem(m_templateTypeAliases, name);
+}
+
 EnumModelItem _ScopeModelItem::findEnum(const QString &name) const
 {
     return findModelItem(m_enums, name);
@@ -897,10 +1038,18 @@ FunctionList _ScopeModelItem::findFunctions(const QString &name) const
 }
 
 // ---------------------------------------------------------------------------
-_NamespaceModelItem::~_NamespaceModelItem()
+_NamespaceModelItem::_NamespaceModelItem(CodeModel *model, int kind)
+    : _ScopeModelItem(model, kind)
 {
 }
 
+_NamespaceModelItem::_NamespaceModelItem(CodeModel *model, const QString &name, int kind)
+    : _ScopeModelItem(model, name, kind)
+{
+}
+
+_NamespaceModelItem::~_NamespaceModelItem() = default;
+
 void _NamespaceModelItem::addNamespace(NamespaceModelItem item)
 {
     m_namespaces.append(item);
@@ -923,11 +1072,31 @@ void  _NamespaceModelItem::appendNamespace(const _NamespaceModelItem &other)
 void _NamespaceModelItem::formatDebug(QDebug &d) const
 {
     _ScopeModelItem::formatDebug(d);
+    switch (m_type) {
+    case NamespaceType::Default:
+        break;
+    case NamespaceType::Anonymous:
+        d << ", anonymous";
+        break;
+    case NamespaceType::Inline:
+        d << ", inline";
+        break;
+    }
     formatScopeList(d, ", namespaces=", m_namespaces);
 }
 #endif // !QT_NO_DEBUG_STREAM
 
 // ---------------------------------------------------------------------------
+_ArgumentModelItem::_ArgumentModelItem(CodeModel *model, int kind)
+    : _CodeModelItem(model, kind)
+{
+}
+
+_ArgumentModelItem::_ArgumentModelItem(CodeModel *model, const QString &name, int kind)
+    : _CodeModelItem(model, name, kind)
+{
+}
+
 _ArgumentModelItem::~_ArgumentModelItem()
 {
 }
@@ -991,6 +1160,16 @@ bool _FunctionModelItem::isSimilar(const FunctionModelItem &other) const
     return true;
 }
 
+_FunctionModelItem::_FunctionModelItem(CodeModel *model, int kind)
+    : _MemberModelItem(model, kind), m_flags(0)
+{
+}
+
+_FunctionModelItem::_FunctionModelItem(CodeModel *model, const QString &name, int kind)
+    : _MemberModelItem(model, name, kind), m_flags(0)
+{
+}
+
 ArgumentList _FunctionModelItem::arguments() const
 {
     return m_arguments;
@@ -1127,6 +1306,20 @@ void _FunctionModelItem::setInvokable(bool isInvokable)
     m_isInvokable = isInvokable;
 }
 
+QString _FunctionModelItem::typeSystemSignature() const  // For dumping out type system files
+{
+    QString result;
+    QTextStream str(&result);
+    str << name() << '(';
+    for (int a = 0, size = m_arguments.size(); a < size; ++a) {
+        if (a)
+            str << ',';
+        m_arguments.at(a)->type().formatTypeSystemSignature(str);
+    }
+    str << ')';
+    return result;
+}
+
 #ifndef QT_NO_DEBUG_STREAM
 void _FunctionModelItem::formatDebug(QDebug &d) const
 {
@@ -1157,6 +1350,16 @@ void _FunctionModelItem::formatDebug(QDebug &d) const
 #endif // !QT_NO_DEBUG_STREAM
 
 // ---------------------------------------------------------------------------
+_TypeDefModelItem::_TypeDefModelItem(CodeModel *model, int kind)
+    : _CodeModelItem(model, kind)
+{
+}
+
+_TypeDefModelItem::_TypeDefModelItem(CodeModel *model, const QString &name, int kind)
+    : _CodeModelItem(model, name, kind)
+{
+}
+
 TypeInfo _TypeDefModelItem::type() const
 {
     return m_type;
@@ -1176,6 +1379,58 @@ void _TypeDefModelItem::formatDebug(QDebug &d) const
 #endif // !QT_NO_DEBUG_STREAM
 
 // ---------------------------------------------------------------------------
+
+_TemplateTypeAliasModelItem::_TemplateTypeAliasModelItem(CodeModel *model, int kind)
+    : _CodeModelItem(model, kind) {}
+
+_TemplateTypeAliasModelItem::_TemplateTypeAliasModelItem(CodeModel *model, const QString &name, int kind)
+    : _CodeModelItem(model, name, kind) {}
+
+TemplateParameterList _TemplateTypeAliasModelItem::templateParameters() const
+{
+    return m_templateParameters;
+}
+
+void _TemplateTypeAliasModelItem::addTemplateParameter(const TemplateParameterModelItem &templateParameter)
+{
+    m_templateParameters.append(templateParameter);
+}
+
+TypeInfo _TemplateTypeAliasModelItem::type() const
+{
+    return m_type;
+}
+
+void _TemplateTypeAliasModelItem::setType(const TypeInfo &type)
+{
+    m_type = type;
+}
+
+#ifndef QT_NO_DEBUG_STREAM
+void _TemplateTypeAliasModelItem::formatDebug(QDebug &d) const
+{
+    _CodeModelItem::formatDebug(d);
+    d << ", <";
+    for (int i = 0, count = m_templateParameters.size(); i < count; ++i) {
+        if (i)
+            d << ", ";
+        d << m_templateParameters.at(i)->name();
+    }
+    d << ">, type=" << m_type;
+}
+#endif // !QT_NO_DEBUG_STREAM
+
+// ---------------------------------------------------------------------------
+_EnumModelItem::_EnumModelItem(CodeModel *model, const QString &name, int kind)
+    : _CodeModelItem(model, name, kind)
+{
+}
+
+_EnumModelItem::_EnumModelItem(CodeModel *model, int kind)
+    : _CodeModelItem(model, kind)
+{
+}
+
 CodeModel::AccessPolicy _EnumModelItem::accessPolicy() const
 {
     return m_accessPolicy;
@@ -1231,6 +1486,16 @@ void _EnumModelItem::formatDebug(QDebug &d) const
 // ---------------------------------------------------------------------------
 _EnumeratorModelItem::~_EnumeratorModelItem() = default;
 
+_EnumeratorModelItem::_EnumeratorModelItem(CodeModel *model, int kind)
+    : _CodeModelItem(model, kind)
+{
+}
+
+_EnumeratorModelItem::_EnumeratorModelItem(CodeModel *model, const QString &name, int kind)
+    : _CodeModelItem(model, name, kind)
+{
+}
+
 QString _EnumeratorModelItem::stringValue() const
 {
     return m_stringValue;
@@ -1252,6 +1517,17 @@ void _EnumeratorModelItem::formatDebug(QDebug &d) const
 // ---------------------------------------------------------------------------
 _TemplateParameterModelItem::~_TemplateParameterModelItem() = default;
 
+_TemplateParameterModelItem::_TemplateParameterModelItem(CodeModel *model, int kind)
+    : _CodeModelItem(model, kind)
+{
+}
+
+_TemplateParameterModelItem::_TemplateParameterModelItem(CodeModel *model,
+                                                         const QString &name, int kind)
+    : _CodeModelItem(model, name, kind)
+{
+}
+
 TypeInfo _TemplateParameterModelItem::type() const
 {
     return m_type;
@@ -1315,6 +1591,16 @@ void _MemberModelItem::setStatic(bool isStatic)
     m_isStatic = isStatic;
 }
 
+_MemberModelItem::_MemberModelItem(CodeModel *model, int kind)
+    : _CodeModelItem(model, kind), m_flags(0)
+{
+}
+
+_MemberModelItem::_MemberModelItem(CodeModel *model, const QString &name, int kind)
+    : _CodeModelItem(model, name, kind), m_flags(0)
+{
+}
+
 bool _MemberModelItem::isConstant() const
 {
     return m_isConstant;
@@ -1423,4 +1709,3 @@ void _MemberModelItem::formatDebug(QDebug &d) const
 #endif // !QT_NO_DEBUG_STREAM
 
 // kate: space-indent on; indent-width 2; replace-tabs on;
-
