@@ -377,7 +377,7 @@ SbkConverter *getConverter(const char *typeName)
     ConvertersMap::const_iterator it = converters.find(typeName);
     if (it != converters.end())
         return it->second;
-    if (Py_VerboseFlag > 0)
+    if (Shiboken::pyVerbose() > 0)
         SbkDbg() << "Can't find type resolver for type '" << typeName << "'.";
     return nullptr;
 }
